#!/usr/bin/env python3
import os
import shutil
from pathlib import Path

import click
import rtoml
from dotenv import load_dotenv
from jinja2 import Environment, FileSystemLoader

load_dotenv()
PACKAGE_DIR = Path(__file__).parent.absolute()
RC_DIR = PACKAGE_DIR / "resources"
PROJECT_DIR = PACKAGE_DIR.parent


class Renderer:
    def __init__(self, searchpath: str | os.PathLike[str]):
        self.searchpath = searchpath
        self.env = Environment(
            loader=FileSystemLoader(self.searchpath),
            trim_blocks=True,
            lstrip_blocks=True,
        )

    def render(
        self,
        template: str | os.PathLike[str],
        output_file: str | os.PathLike[str] = "",
        kwargs: dict | None = None,
    ) -> str:
        t = self.env.get_template(str(template))
        output = t.render(**(kwargs or {}))
        if output_file:
            Path(output_file).write_text(output, encoding="utf-8")
        return output


def process(**kwargs):
    pypjt_version = rtoml.load(PROJECT_DIR / "pyproject.toml")["project"]["version"]

    project_dir = Path(kwargs["project_dir"])
    shutil.copytree(RC_DIR, project_dir)

    (project_dir / "CHANGELOG.md").touch()

    src_dir = project_dir / kwargs["project_name"]
    src_dir.mkdir()
    (src_dir / "__init__.py").touch()
    shutil.copy2(RC_DIR / ".env.template", project_dir / ".env")

    r = Renderer(searchpath=RC_DIR)
    r.render("compose.yml", project_dir / "compose.yml", kwargs)
    r.render("MANIFEST.in", project_dir / "MANIFEST.in", kwargs)
    r.render(
        "pyproject.toml",
        project_dir / "pyproject.toml",
        {**kwargs, "pypjt_version": pypjt_version},
    )
    r.render("README.rst", project_dir / "README.rst", kwargs)
    r.render(".releaserc", project_dir / ".releaserc", kwargs)


@click.command()
def main():
    project_name = click.prompt("Project name", type=str)
    click.echo(f"Project will create at '{Path.cwd() / project_name}'")
    project_dir = Path.cwd() / project_name
    if project_dir.exists():
        raise click.ClickException(
            f"Project dir '{project_dir}' already exists, "
            f"please choose another name or delete it first."
        )
    version = click.prompt("Version", type=str, default="0.0.1")
    author = click.prompt("Author", type=str, default="")
    email = click.prompt("Email", type=str, default="")
    description = click.prompt("Description", type=str, default="")

    process(
        **{
            "project_dir": project_dir,
            "project_name": project_name,
            "version": version,
            "author": author,
            "email": email,
            "description": description,
        }
    )


if __name__ == "__main__":
    main()



import os
import re
import sys

version = os.environ.get("NEW_VERSION")
if not version:
    print("Error: NEW_VERSION environment variable not set.", file=sys.stderr)
    sys.exit(1)

files_to_update = [
    {
        "file": "uv.lock",
        "pattern": re.compile(
            r'(?P<block>\[\[package\]\][^\[]*?name\s*=\s*["\']pypjt["\'][^\[]*?version\s*=\s*["\'])(?P<version>\d+\.\d+\.\d+)(["\'])',
            re.DOTALL,
        ),
        "replacement": lambda m: f"{m.group('block')}{version}{m.group(3)}",
    },
    {
        "file": "pyproject.toml",
        "pattern": re.compile(
            r'(?P<block>\[project\]\s+name\s*=\s*["\'].*?["\']\s+version\s*=\s*["\'])(?P<version>\d+\.\d+\.\d+)(["\'])',
            re.DOTALL,
        ),
        "replacement": lambda m: f"{m.group('block')}{version}{m.group(3)}",
    },
]

print(f"Updating version to: {version}")

for entry in files_to_update:
    filename = entry["file"]
    pattern = entry["pattern"]
    replacement_func = entry["replacement"]

    print(f"Processing: {filename}")

    if not os.path.exists(filename):
        print("File not found, skipping.")
        continue

    try:
        with open(filename, encoding="utf-8") as f:
            content = f.read()

        def replacement_with_log(m, replacement_func=replacement_func):
            old_line = f"{m.group('block')}{m.group('version')}{m.group(3)}"
            new_line = replacement_func(m)
            print(f"Matched:\n{old_line}\nReplaced with:\n{new_line}\n")
            return new_line

        new_content, num_replacements = pattern.subn(replacement_with_log, content)

        if num_replacements > 0:
            with open(filename, "w", encoding="utf-8") as f:
                f.write(new_content)
            print(f"Successfully updated version in {filename}")
        else:
            print("No matching version found or already up-to-date.")
    except Exception as e:
        print(f"Error updating {filename}: {e}", file=sys.stderr)
